<?php
/**
 * Includes CEC Hook Class
 *
 * @author      Murat Purc <murat@purc.de> xmurrix
 * @date        23.04.2008 11:48:34
 * @package     Contenido
 * @subpackage  CEC
 */

/**
 * Static CEC Hook class, which provides a public method to process registered chains at 
 * Contenido Extension Chainer (CEC).
 * 
 * Will only work with chain functions, which accept one argument, multiple arguments 
 * could be passed as a array (assoziative or indexed). Each registered chain function should 
 * return the passed argument.
 * 
 * Usage:
 * <code>
 * // example of executing a cec with multiple parameter
 * $param = array('foo' => $bar);
 * $param = CEC_Hook::execute('Contenido.Content.Somewhere', $param);
 * 
 * // example of executing a cec with a single parameter
 * $foo = 'bar';
 * $foo = CEC_Hook::execute('Contenido.Content.Somewhere', $foo);
  </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @date        23.04.2008 11:48:34
 * @package     Contenido
 * @subpackage  CEC
 */
class CEC_Hook {

    /**
     * Main method to execute registered functions for Contenido Extension Chainer (CEC).
     *
     * Gets the desired CEC iterator and executes each registered chain function
     * 
     * @param   string  $chainName  The chain name to process
     * @param   mixed   $param      Parameter which will be forwarded to registeded chain functions
     * @return  mixed   Parameter changed/processed by chain functions
     */
    function execute($chainName, $param) {

        if (!is_object($GLOBALS['_cecRegistry'])) {
            return $param;
        }
        
        // process CEC
        $_cecIterator = $GLOBALS['_cecRegistry']->getIterator($chainName);
        if ($_cecIterator->count() > 0) {
            $_cecIterator->reset();
            while ($chainEntry = $_cecIterator->next()) {
                $return = $chainEntry->execute($param);
                if (isset($return)) {
                    $param = $return;
                }
            }
        }
        
        return $param;
    }
    
}
