<?php
/**
 * Include class to create websafe names
 *
 * @author      Stefan Seifarth / stese
 * @copyright    www.polycoder.de
 * @author      Murat Purc <murat@purc.de>
 * @date        04.12.2004
 * @date        28.12.2005
 * @package     Contenido
 * @subpackage  ModRewrite
 */

/******************************************
 * File      :   class.modrewrite.php
 * Project   :   Contenido
 * Descr     :   class to create websafe names
 *
 * Author    :   Stefan Seifarth / stese
 * Created   :   04.12.2004
 * Modified  :   28.12.2005
 *
 *  www.polycoder.de
 */


/**
 * Class to create websafe names
 *
 * @author      Stefan Seifarth / stese
 * @package     Contenido
 * @subpackage  ModRewrite
 */
class ModRewrite {

	/**
	 * Constructor Function
	 * @param
	 */
	function ModRewrite() {
		// empty
	} // end function


	function is_enabled(){
		global $cfg;
		return ($cfg['mod_rewrite']['use'] == 1) ? true : false;
	}

	/**
	 * in_categories()
	 *
	 * Check categories on websafe name
	 *
	 * Check all categories in the main parent
	 * category on existing same websafe name
	 *
	 * @param	string	Websafe name to check
	 * @param	integer	current category id
	 * @param	integer current language id
	 * @return 	boolean	true if websafename already exists, false if not
	 */
	function in_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
		global $cfg;

		// get parentid
		$int_parent_id = 0;
		$sql = "SELECT parentid FROM " . $cfg["tab"]["cat"] . " WHERE idcat = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			$int_parent_id = ($aData['parentid'] > 0 ) ? (int) $aData['parentid'] : 0;
		}

		$str_where = " c.parentid = '$int_parent_id' AND";
		$str_where.= " 	cl.idlang = '" . $int_lang_id . "' AND
						cl.urlname = '" . $str_name . "' AND
		cl.idcat <> '$int_id'";

		// check if websafe name is in this category
		$int_count = 0;
		$sql = "SELECT count(cl.idcat) as numcats FROM " . $cfg["tab"]["cat_lang"] . " cl LEFT JOIN " . $cfg["tab"]["cat"] . " c ON cl.idcat = c.idcat WHERE " . $str_where;
		if ($aData = mr_query_n_next_record($sql)) {
			return ($aData['numcats'] > 0) ? true : false;
		}

		return false;
	} // end function


	/**
	 * in_articles()
	 *
	 * Check articles on websafe name
	 *
	 * Check all articles in the current category
	 * on existing same websafe name
	 *
	 * @param	string	Websafe name to check
	 * @param	integer	current article id
	 * @param	integer current language id
	 * @return 	boolean	true if websafename already exists, false if not
	 */
	function in_articles ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
		global $cfg;

		// handle multipages
		$int_category_id = 0;
		if ($int_idcat > 0) {
			$int_category_id = $int_idcat;
		} else {
			// get category id if not set
			$sql = "SELECT idcat FROM " . $cfg["tab"]["cat_art"] . " WHERE idart = '$int_id'";
			if ($aData = mr_query_n_next_record($sql)) {
				$int_category_id = ($aData['idcat'] > 0 ) ? (int) $aData['idcat'] : 0;
			}
		}

		$str_where = " ca.idcat = '$int_category_id' AND";
		$str_where.= " 	al.idlang = '" . $int_lang_id . "' AND
						al.urlname = '" . $str_name . "' AND
		al.idart <> '$int_id'";

		// check if websafe name is in this category
		$sql = "SELECT count(al.idart) as numcats FROM " . $cfg["tab"]["art_lang"] . " al LEFT JOIN " . $cfg["tab"]["cat_art"] . " ca ON al.idart = ca.idart WHERE " . $str_where;
		if ($aData = mr_query_n_next_record($sql)) {
			return ($aData['numcats'] > 0) ? true : false;
		}

		return false;
	} // end function


	/**
	 * set_article()
	 *
	 * set websafe name in article list
	 *
	 * insert new websafe name in article list
	 *
	 * @param	string	original name (will be converted)
	 * @param	integer	current article id
	 * @param	integer current language id
	 * @return 	boolean	true if insert was successfully
	 */
	function set_article ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
		global $cfg;

		// create websafe name
		$str_new_name = capiStrCleanURLCharacters(html_entity_decode($str_name));

		// only html files part
		if (ModRewrite::validate_setting_categories_as_html()) {
			$str_new_name = str_replace( $cfg["mod_rewrite"]['article_seperator'], $cfg["mod_rewrite"]['article_word_seperator'], $str_new_name );
		}

		// check for double word seperators
		if ( strlen(trim($cfg["mod_rewrite"]['article_word_seperator'])) > 0 ) {
			while ( ereg($cfg["mod_rewrite"]['article_word_seperator'].$cfg["mod_rewrite"]['article_word_seperator'],$str_new_name) ) {
				$str_new_name = str_replace($cfg["mod_rewrite"]['article_word_seperator'].$cfg["mod_rewrite"]['article_word_seperator'], $cfg["mod_rewrite"]['article_word_seperator'],$str_new_name);
			}
		}

		// check if websafe name already exists
		if ( ModRewrite::in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat) ) {
			// create new websafe name if exists
			$str_new_name = $str_new_name . $int_id;
		}

		// check again - and set name
		if (!ModRewrite::in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat)) {
				
			// insert websafe name in article list
			$db = new DB_Contenido;
			$sql = "UPDATE " . $cfg["tab"]["art_lang"] . " SET urlname = '$str_new_name' WHERE idart = '$int_id' AND idlang = '$int_lang_id'";
			return $db->query($sql);
				
		} else {
			return false;
		}
	} // end function


	/**
	 * set_category()
	 *
	 * set websafe name in category list
	 *
	 * insert new websafe name in category list
	 *
	 * @param	string	original name (will be converted)
	 * @param	integer	current article id
	 * @param	integer current language id
	 * @return 	boolean	true if insert was successfully
	 */
	function set_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
		global $cfg;

		// create websafe name
		$str_new_name = capiStrCleanURLCharacters(html_entity_decode($str_name));

		// only html files part
		if (ModRewrite::validate_setting_categories_as_html()) {
			$str_new_name = str_replace( $cfg["mod_rewrite"]['category_seperator'], $cfg["mod_rewrite"]['category_word_seperator'], $str_new_name );
			$str_new_name = str_replace( $cfg["mod_rewrite"]['article_seperator'], $cfg["mod_rewrite"]['category_word_seperator'], $str_new_name );
		}

		// check for double word seperators
		if ( strlen(trim($cfg["mod_rewrite"]['category_word_seperator'])) > 0 ) {
		 while ( ereg($cfg["mod_rewrite"]['category_word_seperator'].$cfg["mod_rewrite"]['category_word_seperator'],$str_new_name) ) {
		 	$str_new_name = str_replace($cfg["mod_rewrite"]['category_word_seperator'].$cfg["mod_rewrite"]['category_word_seperator'], $cfg["mod_rewrite"]['category_word_seperator'],$str_new_name);
		 }
		}

		// check if websafe name already exists
		if (ModRewrite::in_category($str_new_name, $int_id, $int_lang_id)) {
			// create new websafe name if exists
			$str_new_name = $str_new_name . $int_id;
		}


		// check again - and set name
		if (!ModRewrite::in_category($str_new_name, $int_id, $int_lang_id)) {
				
			// insert websafe name in article list
			$db = new DB_Contenido;
			$sql = "UPDATE " . $cfg["tab"]["cat_lang"] . " SET urlname = '$str_new_name' WHERE idcat = '$int_id' AND idlang = '$int_lang_id'";
			return $db->query($sql);
				
		} else {
			return false;
		}
	} // end function


	/**
	 * get_id_from_idartlang()
	 *
	 * get article id and language id from article language id
	 *
	 * @param	integer	current article id
	 * @return 	Array	Array with idart and idlang of current article
	 */
	function get_id_from_idartlang ( $int_id = 0 ) {
		global $cfg;
		$sql = "SELECT idart, idlang FROM " . $cfg["tab"]["art_lang"] . " WHERE idartlang = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData;
		}
		return array();
	}	// end function


	/**
	 * get_catname()
	 *
	 * get category name from category id and language id
	 *
	 * @param	integer	category id
	 * @param	integer	language id
	 * @return 	string	category name
	 */
	function get_catname ( $int_id = 0, $int_lang_id = 0 ) {
		global $cfg;
		$sql = "SELECT name FROM " . $cfg["tab"]["cat_lang"] . " WHERE idcat = '$int_id' AND idlang = '$int_lang_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData['name'];
		}
		return '';
	}	// end function


	/**
	 * get_arttitle()
	 *
	 * get article name from article id and language id
	 *
	 * @param	integer	article id
	 * @param	integer	language id
	 * @return 	string	article name
	 */
	function get_arttitle ( $int_id = 0, $int_lang_id = 0 ) {
		global $cfg;
		$sql = "SELECT title FROM " . $cfg["tab"]["art_lang"] . " WHERE idart = '$int_id' AND idlang = '$int_lang_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData['title'];
		}
		return '';
	}	// end function


	/**
	 * get_cat_languages()
	 *
	 * get language ids from category id
	 *
	 * @param	integer	category id
	 * @return 	array	used language ids
	 */
	function get_cat_languages ( $int_id = 0 ) {
		global $cfg;

		$db = new DB_Contenido;
		$sql = "SELECT idlang FROM " . $cfg["tab"]["cat_lang"] . " WHERE idcat = '$int_id'";
		$db->query($sql);

		$arr_languages = Array();
		while ($db->next_record()) {
			$arr_languages[] = $db->f("idlang");
		}

		return $arr_languages;

	}	// end function


	/**
	 * get_artids()
	 *
	 * get article title, language id
	 *
	 * @param	integer	idartlang
	 * @return 	array	title, idlang
	 */
	function get_artids ( $int_id = 0 ) {
		global $cfg;
		$sql = "SELECT urlname, idlang FROM " . $cfg["tab"]["art_lang"] . " WHERE idartlang = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData;
		}
		return array();
	}	// end function


	/**
	 * build_recursiv_path()
	 *
	 * build a recursiv path for mod_rewrite rule like
	 * server directories ( dir1/dir2/dir3 )
	 *
	 * @param	integer	latest category id
	 * @return 	string	linkpath with correct uri
	 */
	function build_recursiv_path ( $int_id = 0, $int_lang_id = 0, $int_lastid = 0 ) {
		global $cfg;

		$arr_directories = Array();
		$bool_finish = false;
		$int_tmp_idcat = $int_id;
		$str_join_parameter = '/';

		while ($bool_finish == false ) {
			$sql = "SELECT cl.urlname, c.parentid
					FROM " . $cfg["tab"]["cat_lang"] . " cl 
					LEFT JOIN  " . $cfg["tab"]["cat"] . " c
			ON cl.idcat = c.idcat
			WHERE cl.idcat = '$int_tmp_idcat'
			AND cl.idlang = '$int_lang_id'";
			if ($aData = mr_query_n_next_record($sql)) {
				$arr_directories[] = $aData['urlname'];
				$int_tmp_idcat     = $aData['parentid'];

				if ($aData['parentid'] == 0 || $aData['parentid'] == $int_lastid) {
					$bool_finish = true;
				}
			} else {
				$bool_finish = true;
			}
		}

		// only html files part
		if (ModRewrite::validate_setting_categories_as_html()) {
			$str_join_parameter = $cfg["mod_rewrite"]['category_seperator'];
		}

		// reverse array entries and create directory string
		$str_path = join($str_join_parameter,array_reverse($arr_directories));

		return $str_path;

	}	// end function


	/**
	 * return full contenido url from single anchor
	 *
	 * @param array $arr_matches [0] = complete anchor, [1] = pre arguments, [2] = anchor name, [3] = post arguments
	 * @return string new anchor
	 */
	function rewrite_html_anchor( $arr_matches = array() ) {
		global $parts, $idcat, $idart, $artname, $client, $lang, $sess;

		// set article name
		$str_idart = '';
		if (isset ( $artname ) && strlen($artname) > 0 ) {
			$str_idart = '&idart=' . $idart;
		}

		// check for additional parameter in url
		$arr_ignored_params = array ( 'idcat', 'idart', 'lang', 'client', 'idcatart', 'changelang', 'changeclient', 'idartlang', 'parts', 'artname' );
		$str_other_params = '';

		//print_r ( $arr_matches);

		if (isset($_GET) && count($_GET)>0 ) {
			//print_r ( $_GET);

			foreach ( $_GET as $str_key => $str_value) {
				if (!in_array($str_key, $arr_ignored_params) && strlen(trim($str_value)) > 0 ) {
					$arr_no_anchor = explode("#",$str_value);

					$str_other_params .= '&' . urlencode(urldecode($str_key)) . '=' . urlencode(urldecode($str_value));
				}
			}
		}
		$str_new_url = '<a' . $arr_matches[1] . 'href="' . $sess->url(
			'front_content.php?' .
			'idcat=' . $idcat .
			'&client=' . $client .
			'&changelang=' . $lang .
		$str_idart .
		$str_other_params .
			'#' . $arr_matches[2]
		) . '"' . $arr_matches[3] . '>';

		return $str_new_url;
	}


	/**
	 * return full contenido url from single anchor
	 *
	 * @param array $arr_matches [0] = complete anchor, [1] = pre arguments, [2] = anchor name, [3] = post arguments
	 * @return string new anchor
	 */
	function contenido_html_anchor( $arr_matches = array(), $str_xhtml = true ) {
		global $parts, $idcat, $idart, $artname, $client, $lang, $sess;

		$arr_params = array();
		$str_join_parameter = $str_xhtml ? '&amp;' : '&';

		foreach ( $_GET as $str_key => $str_value ) {
			$arr_no_anchor = explode("#",$str_value);
			$arr_params[] = urlencode(urldecode($str_key)) . '=' . urlencode(urldecode($arr_no_anchor[0]));
		}

		$str_new_url = '<a' . $arr_matches[1] . 'href="' . $sess->url( 'front_content.php?' . implode($str_join_parameter, $arr_params) . '#' . $arr_matches[2] ) . '"' . $arr_matches[3] . '>';

		return $str_new_url;
	}


	/**
	 * build_new_url()
	 *
	 * build new url from given arguments
	 *
	 * get querystring of front_content.php and
	 * convert this url to the new mod_rewrite url
	 * method will be startet before the complete
	 * output of the front site will be executed
	 *
	 * @param   string   given arguments
	 * @return    string   new url
	 * @modified Stefan Seifarth 2005-08-14
	 */
	function build_new_url ( $str_args = "", $str_xhtml = true ) {
		global $cfg, $lang, $client;

		$str_new_url = "";
		$str_categories = "";
		$str_article = "";
		$arr_args = array();
		$str_join_parameter = "/";
		$str_file_extension = "";
		$arr_parts = array();

		// only html files part
		if (ModRewrite::validate_setting_categories_as_html()) {
			$str_join_parameter = $cfg['mod_rewrite']['category_seperator'];
			$str_file_extension = $cfg['mod_rewrite']['file_extension'];
		}

		// check arguments ... and split
		$str_anchor = "";
		// check for anchor
		$arr_args = explode("#", $str_args);
		if (is_array($arr_args) && count($arr_args)>1) {
			$str_args = $arr_args[0];
			$str_anchor = '#' . urlencode(urldecode($arr_args[1]));
		}

		$str_args = str_replace('?', '', $str_args);
		$str_args = str_replace('&amp;', '&', $str_args);
		// extract arguments into current scope
		parse_str ($str_args);

		// some preparations to avoid too much checks in further code blocks
		$idcat        = (isset($idcat)) ? (int) $idcat : 0;
		$idart        = (isset($idart)) ? (int) $idart : 0;
		$idcatart     = (isset($idcatart)) ? (int) $idcatart : 0;
		$idartlang    = (isset($idartlang)) ? (int) $idartlang : 0;
		$changelang   = (isset($changelang)) ? (int) $changelang : 0;
		$changeclient = (isset($changeclient)) ? (int) $changeclient : 0;

		// get additional non contenido parameters
		$str_additional_params = '';
		$arr_additional_params = array();
		$arr_param_pairs = split( "&", $str_args );

		$arr_ignored_params = array ( 'idcat', 'idart', 'lang', 'client', 'idcatart', 'changelang', 'changeclient', 'idartlang' );

		foreach ( $arr_param_pairs as $str_pair ) {
			$bol_found_bad = false;
			$arr_param = split ( "=", $str_pair );

			foreach ( $arr_ignored_params as $str_key ) {
				if ( $str_key == strtolower(trim($arr_param[0])) ) {
					$bol_found_bad = true;
					break;
				}
			}

			if ( $bol_found_bad == false ) {
				$arr_additional_params[] = urlencode(urldecode($arr_param[0])) . '=' . urlencode(urldecode($arr_param[1]));
			}
		}

		if ( count ( $arr_additional_params ) > 0 ) {
			$str_glue = ( $str_xhtml == true ) ? '&amp;' : '&';
			$str_additional_params = '?' . implode ( $str_glue, $arr_additional_params );
		}

		$idlang   = ($changelang > 0) ? $changelang : $lang;
		$idclient = ($changeclient > 0) ? $changeclient : $client;

		// set client?
		if ( $cfg['mod_rewrite']['use_client'] == 1 ) {
			if ( $cfg['mod_rewrite']['use_client_name'] == 1 ) {
				$arr_parts[] = urlencode(ModRewrite::get_client_name($idclient));
			} else {
				$arr_parts[] = $idclient;
			}
		}

		// set language?
		if ( $cfg['mod_rewrite']['use_language'] == 1 ) {
			if ( $cfg['mod_rewrite']['use_language_name'] == 1 ) {
				$arr_parts[] = urlencode(ModRewrite::get_language_name($idlang));
			} else {
				$arr_parts[] = $idlang;
			}
		}

		// define rootpath ...
		$str_new_url = $cfg['mod_rewrite']['rootdir'];

		// handle idcatart
		if ($idcatart > 0 && $idcat == 0 && $idart == 0 && $idartlang == 0) {
			$sql = "SELECT idcat, idart FROM " . $cfg["tab"]["cat_art"] . " WHERE idcatart = '$idcatart'";
			if ($aData = mr_query_n_next_record($sql)) {
				$idcat = $aData['idcat'];
				$idart = $aData['idart'];
			}
		}

		// check if article id is given and set article url
		if ($idart > 0 && $idartlang == 0) {
			$str_article = ModRewrite::get_art_websafename ( $idart, $idlang );
		} elseif ($idartlang > 0 && $idart == 0) {
			// handle idartlang
			$str_article = ModRewrite::get_art_lang_websafename ( $idartlang );
		}

		// check if only article id is given, cat id have to rebuild
		if ($idart > 0 && $idcat == 0 && $idartlang == 0) {
			$sql = "SELECT idcat FROM " . $cfg["tab"]["cat_art"] . " WHERE idart = '$idart'";
			if ($aData = mr_query_n_next_record($sql)) {
				$idcat = $aData['idcat'];
			}
		}

		// idartlang is given
		if ($idartlang > 0 && $idcat == 0 && $idart == 0) {
			$sql = "SELECT ca.idcat
					FROM " . $cfg["tab"]["art_lang"] . " al 
					LEFT JOIN " . $cfg["tab"]["cat_art"] . " ca ON al.idart = ca.idart
			WHERE al.idartlang = '$idartlang'";
			if ($aData = mr_query_n_next_record($sql)) {
				$idcat = $aData['idcat'];
			}
		}
		
		// we have category but no article name and start article name should be added to url
		if ($idcat > 0 && $str_article == '' && $cfg['mod_rewrite']['add_startart_name_to_url']) {
			if ($cfg['mod_rewrite']['default_startart_name'] == '') {

				// no default article name is configured get startarticle
				cInclude('classes', 'class.article.php');
				$artColl = new ArticleCollection(array('idcat' => $idcat, 'start' => 1));
				if ($artItem = $artColl->startArticle()) {
					$idart       = $artItem->get('idart');
					$str_article = ModRewrite::get_art_websafename($idart, $idlang);
				}
				
			} else {

				// use default start article name
				$str_article = $cfg['mod_rewrite']['default_startart_name'];
			}
		}

		if ($str_article) {
			$str_file_extension = $cfg['mod_rewrite']['file_extension'];
		}
		
		// ok build dir list, if idcat found ...
		if ($idcat > 0) {

			$str_categories = ModRewrite::build_recursiv_path ( $idcat, $idlang );

			// check start directory settings
			if ( $cfg['mod_rewrite']['startfromroot'] == 0 ) {
				// splitt string in array
				$arr_categories = split( $str_join_parameter, $str_categories);

				// remove first category
				$str_first_cat = array_shift ( $arr_categories );

				// implode array with categories to new string
				$str_categories = join ( $str_join_parameter,  $arr_categories );
			}

			if (strlen($str_categories)> 0) {
				$arr_parts[] = $str_categories;
			}
		}
		$str_parts = implode( $str_join_parameter, $arr_parts);

		// only html files part
		if (strlen($str_parts) > 0 && strlen($str_article) > 0
		&& ModRewrite::validate_setting_categories_as_html()) {
			$str_parts.= $cfg['mod_rewrite']['article_seperator'];
		} else if ((int)$cfg['mod_rewrite']['use_categories_as_html_file'] != 1 && strlen($str_parts) > 0) {
			$str_parts.= $str_join_parameter;
		}

		// using lowercase uri?
		if ( $cfg['mod_rewrite']['use_lowercase_uri'] == 1 ) {
			$str_full_url = $str_new_url . strtolower( $str_parts . $str_article . $str_file_extension ) . $str_additional_params . $str_anchor;
		} else {
			$str_full_url = $str_new_url . $str_parts . $str_article . $str_file_extension . $str_additional_params . $str_anchor;
		}

		// remove double slashes
		while ( eregi($str_join_parameter.$str_join_parameter, $str_full_url) ) {
			$str_full_url = eregi_replace ( $str_join_parameter.$str_join_parameter, $str_join_parameter, $str_full_url );
		}

		if (substr ($str_full_url, -2) == "?=") {
			$str_full_url = substr_replace($str_full_url, "", -2);
		}

		return $str_full_url;

	} // end function

	 
	/**
	 * get_art_websafename()
	 *
	 * get article websafe name from article id and language id
	 *
	 * @param	integer	article id
	 * @param	integer	language id
	 * @return 	string	article websafe name
	 */
	function get_art_websafename ( $int_id = 0, $int_lang_id = 0 ) {
		global $cfg;
		$sql = "SELECT urlname FROM " . $cfg["tab"]["art_lang"] . " WHERE idart = '$int_id' AND idlang = '$int_lang_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return urldecode($aData['urlname']);
		}
		return null;
	}	// end function
	 
	   
	/**
	 * get_art_lang_websafename()
	 *
	 * get article websafe name from idartlang
	 *
	 * @param	integer	idartlang
	 * @return 	string	article websafe name
	 */
	function get_art_lang_websafename ( $int_id = 0 ) {
		global $cfg;
		$sql = "SELECT urlname FROM " . $cfg["tab"]["art_lang"] . " WHERE idartlang = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return urldecode($aData['urlname']);
		}
		return null;
	}	// end function


	/**
	 * get_client_name()
	 *
	 * get name of client id
	 *
	 * @param	integer	language id
	 * @return 	string	language name
	 */
	function get_client_name ( $int_id = 0 ) {
		global $cfg;
		$sql = "SELECT name FROM " . $cfg["tab"]["clients"] . " WHERE idclient = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData['name'];
		}
		return null;
	}	// end function


	/**
	 * get_language_name()
	 *
	 * get name of language id
	 *
	 * @param	integer	language id
	 * @return 	string	language name
	 */
	function get_language_name_old ( $int_id = 0 ) {
		global $cfg;
		$sql = "SELECT name FROM " . $cfg["tab"]["lang"] . " WHERE idlang = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			return $aData['name'];
		}
		return null;
	}	// end function

	
	/**
	 * Returns name of language
	 *
	 * @param   int   $int_id  Language id
	 * @return  array  Assoziative array containing lanuage names ($a[id] = 'name')
	 */
	function get_language_name ( $int_id = 0 ) {
		global $cfg;
		static $aName;
		if (!isset($aName)) {
			$aName = array();
		}

		if (isset($aName[$int_id])) {
			return $aName[$int_id];
		}


		$sql = "SELECT name FROM " . $cfg["tab"]["lang"] . " WHERE idlang = '$int_id'";
		if ($aData = mr_query_n_next_record($sql)) {
			$aName[$int_id] = $aData['name'];
		} else {
			$aName[$int_id] = '';
		}

		return $aName[$int_id];
	}	// end function


	/**
	 * get_client_full_url_parts()
	 * Splits passed argument into scheme://host and path/query.
	 *
	 * Example:
	 * input  = http://host/front_content.php?idcat=123
	 * return = array('htmlpath' => 'http://host', 'url' => 'front_content.php?idcat=123')
	 *
	 * @param  string  $url  URL to split
	 * @return array  Assoziative array including the two parts:
	 *                - array('htmlpath' => $path, 'url' => $url)
	 */
	function get_client_full_url_parts($url) {
		global $cfg, $cfgClient, $client;
		$clientPath = $cfgClient[$client]['path']['htmlpath'];

		if (stristr($url, $clientPath) !== false) {

			// url includes full html path (scheme host path, etc.)
			$url      = str_replace($clientPath, '', $url);
			$htmlPath = $clientPath;
			$aComp    = parse_url($htmlPath);

			// check if path matches to defined rootdir from mod_rewrite conf
			if (isset($aComp['path']) && $aComp['path'] !== $cfg['mod_rewrite']['rootdir']) {
				// replace not matching path agaings configured one
				// this will replace e. g. "http://host/cms/" against "http://host/"
				$htmlPath = str_replace($aComp['path'], $cfg['mod_rewrite']['rootdir'], $htmlPath);
				if (substr($htmlPath, strlen($htmlPath)-1) == '/') {
					// remove last slash
					$htmlPath = substr($htmlPath, 0, strlen($htmlPath)-1);
				}
			}
		} else {
			$htmlPath = '';
		}
		return array('htmlpath' => $htmlPath, 'url' => $url);
	}


	/**
	 * reset_categories_aliases()
	 *
	 * method to reset all aliases in categories
	 */
	function reset_categories_aliases() {
		global $cfg;

		$db = new DB_Contenido();

		// empty all aliases
		$sql = "UPDATE " . $cfg["tab"]["cat_lang"] . " SET urlname = ''";
		$db->query($sql);

		// get all categories
		$sql = "SELECT name, idcat, idlang FROM " . $cfg["tab"]["cat_lang"];
		$db->query($sql);

		while ( $db->next_record() ) {
			//set new alias
			ModRewrite::set_category($db->f('name'), $db->f('idcat'), $db->f('idlang'));
		}
		unset ($db);
	}


	/**
	 * reset_articles_aliases()
	 *
	 * method to reset all aliases in articles
	 */
	function reset_articles_aliases () {
		global $cfg;
		
		$db = new DB_Contenido();

		// empty all aliases
		$sql = "UPDATE " . $cfg["tab"]["art_lang"] . " SET urlname = ''";
		$db->query($sql);

		// get all articles
		$sql = "SELECT title, idart, idlang FROM " . $cfg["tab"]["art_lang"];
		$db->query($sql);

		while ( $db->next_record() ) {
			//set new alias
			ModRewrite::set_article($db->f('title'), $db->f('idart'), $db->f('idlang'));
		}
		
		unset ($db);
	}

	/**
	 * reset_aliases()
	 *
	 * method to reset all aliases (category and article)
	 */
	function reset_aliases () {
		ModRewrite::reset_categories_aliases();
		ModRewrite::reset_articles_aliases();
	}


	/**
	 * Used to postprocess resolved path
	 *
	 * Error site handling if category not found
	 *
	 * if percentage == 100 and there is no 100 percentage result value,
	 * error site will be shown - can be adjust by user settings for
	 * smooth similar effects - 80 to 95 will be best but have to check by user
	 *
	 * @param   array  $results  Pathresolver results array
	 * @return  mixed  Categoryid or false
	 */
	function get_id_from_pathresolver_result($results) {
		global $cfg;

		$intMinPercentage = $cfg["mod_rewrite"]['category_resolve_min_percentage'];
		$catId = key($results);

		if (isset($intMinPercentage) && (int)$intMinPercentage > 0 && $results[$catId] < $intMinPercentage) {
			return false;
		} else {
			return $catId;
		}
	}


	/**
	 * Analyses the settings for usage of categories as a html file and returns
	 * the result as a boolean
	 *
	 * @return  bool  True if settings are active and valid otherwhise false.
	 */
	function validate_setting_categories_as_html() {
		global $cfg;
		
		if ((int) $cfg["mod_rewrite"]['use_categories_as_html_file'] == 1
			&& strlen($cfg["mod_rewrite"]['article_seperator']) > 0
			&& strlen($cfg["mod_rewrite"]['category_seperator']) > 0
			&& $cfg["mod_rewrite"]['category_seperator'] != $cfg["mod_rewrite"]['article_seperator']) {
			// settings are ok, return true
			return true;
		} else {
			// settings are are deactivated or faulty
			return false;
		}
	}

} // end class
