<?php
/**
 * Defines the 'modrewrite' related functions
 *
 * @author      Stefan Seifarth / stese
 * @copyright    www.polycoder.de
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  ModRewrite
 */

/******************************************
 * File      :   functions.mod_rewrite.php
 * Project   :   Contenido
 * Descr     :   Defines the 'modrewrite' related
 *               functions
 *
 * Author    :   Stefan Seifarth
 * Created   :   04.12.2004
 * Modified  :   18.12.2005
 *
 *  www.polycoder.de
 ******************************************/


/**
 * mr_get_language_id()
 *
 * get language id from language name
 * thanks to Nicolas Dickinson for multi
 * Client/Language BugFix
 *
 * @param	string	language name
 * @return integer	language id
 */
function mr_get_language_id ( $str_languagename = "", $int_client = 1 ) {
	global $cfg;

	$int_lang_id = false;

	$sql = "SELECT l.idlang
           FROM " . $cfg["tab"]["lang"] . " as l 
           LEFT JOIN " . $cfg["tab"]["clients_lang"] . " AS cl ON l.idlang = cl.idlang
           WHERE
              cl.idclient = '". (int)$int_client . "' AND 
              l.name = '" . urldecode($str_languagename) . "'";
	if ($aData = mr_query_n_next_record($sql)) {
		$int_lang_id = $aData['idlang'];
	}

	return $int_lang_id;
}


/**
 * mr_get_client_id()
 *
 * get client id from client name
 *
 * @param	  string    client name
 * @return  integer   client id
 */
function mr_get_client_id ( $str_clientname = "" ) {
	global $cfg;

	$int_client_id = false;

	$sql = "SELECT idclient
			FROM " . $cfg["tab"]["clients"] . "
			WHERE name = '" . urldecode($str_clientname) . "'";
	if ($aData = mr_query_n_next_record($sql)) {
		$int_client_id = $aData['idclient'];
	}

	return $int_client_id;
}


/**
 * mr_get_idart()
 *
 * get article id
 *
 * get recent article from websafe name tree
 *
 * @modified: 2005-09-11
 * @param	string	Websafe name
 * @param	integer	category id
 * @return 	integer	recent article id
 */
function mr_get_idart ( $str_artname = "", $int_id = 0, $int_lang_id = 0 ) {
	global $cfg;
	static $db;

	$int_idart = false;

	if (!isset($db)) {
		$db = new DB_Contenido;
	}

	$str_where = "";
	// only article name were given
	if ($int_id == 0) {
		// get all basic category ids with parentid=0
		$sql = "SELECT idcat FROM " . $cfg["tab"]["cat"] . " WHERE parentid = '0'";
		$db->query ( $sql );

		$arr_idcats = array();
		$arr_where = array();

		while ($db->next_record()) {
			$arr_idcats[] = "idcat = '" . $db->f("idcat") . "'";
		}
		$str_where = " AND ( " . join(" OR ", $arr_idcats) . ")";
	} else {
		$str_where = " AND ca.idcat = '$int_id'";
	}

	$sql = "SELECT al.idart
			FROM " . $cfg["tab"]["art_lang"] . " al 
			LEFT JOIN " . $cfg["tab"]["cat_art"] . " ca 
	ON al.idart = ca.idart
	WHERE al.urlname = '$str_artname'" . $str_where;

	if ($aData = mr_query_n_next_record($sql)) {
		$int_idart = $aData['idart'];
	}

	return $int_idart;
}


/**
 * mr_get_setting_override()
 *
 * override local cfg settings with client settings from database
 *
 * @param	integer	client id
 */
function mr_get_setting_override ( $int_client_id ) {
	// TODO: we don't relly need this!
	return;
	global $cfg;
	static $db;

	if (!isset($db)) {
		$db = new DB_Contenido;
	}

	$sql = "SELECT name, value FROM " . $cfg["tab"]["properties"] . " WHERE type='ModRewrite' AND idclient = '" . $int_client_id . "' AND name NOT LIKE 'frontend_path%'";
	$db->query($sql);
	 
	while ( $db->next_record() ) {
		$cfg['mod_rewrite'][urldecode($db->f("name"))] = urldecode($db->f("value"));
	}
}


// plugin Advanced Mod Rewrite - Murat Purc (aka xmurrix)

/**
 * Processes mod_rewrite related job after moving a category up.
 *
 * Will be called as a action code from table con_actions.
 *
 * @param  int  $idcat  Id of category beeing moved up
 */
function mr_strMoveUpCategory($idcat) {

	// category check
	$cat = new cApiCategory($idcat);
	if (!$cat->get('preid')) {
		return;
	}

	// get all cat languages
	$arr_idlang = ModRewrite::get_cat_languages($idcat);

	// update ...
	foreach ($arr_idlang as $int_idlang) {
		// get urlname
		$str_catname = ModRewrite::get_catname($idcat, $int_idlang);
		// set new urlname - because original set urlname isn't validated for double entries in same parent category
		ModRewrite::set_category($str_catname, $idcat, $int_idlang);
	}

}


/**
 * Processes mod_rewrite related job after moving a category down.
 *
 * Will be called as a action code from table con_actions.
 *
 * @param  int  $idcat  Id of category beeing moved down
 */
function mr_strMoveDownCategory($idcat) {

	// category check
	$cat = new cApiCategory($idcat);
	if (!$cat->get('id')) {
		return;
	}

	// get all cat languages
	$arr_idlang = ModRewrite::get_cat_languages($idcat);
	// update ...
	foreach ($arr_idlang as $int_idlang) {
		// get urlname
		$str_catname = ModRewrite::get_catname($idcat, $int_idlang);
		// set new urlname - because original set urlname isn't validated for double entries in same parent category
		ModRewrite::set_category($str_catname, $idcat, $int_idlang);
	}

}



/**
 * Processes mod_rewrite related job after moving a category subtree.
 *
 * Will be called as a action code from table con_actions.
 *
 * @param  int  $idcat  Id of category where the subtree has beeen moved
 */
function mr_strMoveSubtree($idcat) {
	// category check
	$cat = new cApiCategory($idcat);
	if (!$cat->get('id')) {
		return;
	}

	// get all cat languages
	$arr_idlang = ModRewrite::get_cat_languages($idcat);
	// update ...
	foreach ($arr_idlang as $int_idlang) {
		// get urlname
		$str_catname = ModRewrite::get_catname($idcat, $int_idlang);
		// set new urlname - because original set urlname isn't validated for double entries in same parent category
		ModRewrite::set_category($str_catname, $idcat, $int_idlang);
	}
}


/**
 * Processes mod_rewrite related job after editing a article first time.
 *
 * Will be called as a action code from table con_actions.
 *
 * @param  int  $newIdart  Id of new edited article
 */
function mr_conEditFirstTime($newIdart) {
	global $client, $title, $urlname, $idcat;

	if (strlen(trim($urlname)) == 0) {
		$urlname = $title;
	}

	$a_languages = getLanguagesByClient($client);

	foreach ($a_languages as $tmp_lang) {
		ModRewrite::set_article($urlname, $newIdart, $tmp_lang, $idcat);
	}
}


/**
 * Processes mod_rewrite related job after editing a article.
 *
 * Will be called as a action code from table con_actions.
 *
 * @param  int  $idart  Id of edited article
 */
function mr_conEditArt($idart){
	global $urlname, $title, $idartlang;

	if (strlen(trim($urlname)) == 0) {
		$urlname = $title;
	}

	$arr_art = ModRewrite::get_id_from_idartlang($idartlang);
	if (count($arr_art) == 2) {
		ModRewrite::set_article($urlname, $arr_art['idart'], $arr_art['idlang'], $idcat);
	}
}


/**
 * Processes mod_rewrite related job for articles beeing moved.
 *
 * Will be called by chain 'Contenido.Article.conMoveArticles_Loop'.
 *
 * @param   array  $param  Assoziative array with record entries
 * @return  array  Loop through of arguments
 */
function mr_conMoveArticles($param){
	// too defensive but secure way
	if (!is_array($param)) {
		return $param;
	} elseif (!isset($param['idartlang'])) {
		return $param;
	} elseif (!isset($param['idart'])) {
		return $param;
	}

	$arr_art = ModRewrite::get_artids($param['idartlang']);
	if (count($arr_art) == 2) {
		ModRewrite::set_article($arr_art["urlname"], $param['idart'], $arr_art["idlang"]);
	}

	return $param;
}


/**
 * Processes mod_rewrite related job for duplicated articles.
 *
 * Will be called by chain 'Contenido.Article.conCopyArtLang_AfterInsert'.
 *
 * @param   array  $param  Assoziative array with record entries
 * @return  array  Loop through of arguments
 */
function mr_conCopyArtLang($param) {
	// too defensive but secure way
	if (!is_array($param)) {
		return $param;
	} elseif (!isset($param['title'])) {
		return $param;
	} elseif (!isset($param['idart'])) {
		return $param;
	} elseif (!isset($param['idlang'])) {
		return $param;
	}

	ModRewrite::set_article($param['title'], $param['idart'], $param['idlang']);

	return $param;
}


/**
 * Processes mod_rewrite related job for synchronized articles.
 *
 * Will be called by chain 'Contenido.Article.conSyncArticle_AfterInsert'.
 *
 * @param   array  $param  Assoziative array with record entries as follows:
 * <code>
 * array(
 *     'src_art_lang'  => Recordset (assoziative array) of source item from con_art_lang table
 *     'dest_art_lang' => Recordset (assoziative array) of inserted destination item from con_art_lang table
 * );
 * </code>
 *
 * @return  array  Loop through of argument
 */
function mr_conSyncArticle($param){
	// too defensive but secure way
	if (!is_array($param)) {
		return $param;
	} elseif (!isset($param['src_art_lang']) || !is_array($param['src_art_lang'])) {
		return $param;
	} elseif (!isset($param['dest_art_lang']) || !is_array($param['dest_art_lang'])) {
		return $param;
	} elseif (!isset($param['dest_art_lang']['idart'])) {
		return $param;
	} elseif (!isset($param['dest_art_lang']['idlang'])) {
		return $param;
	}

	if (!isset($param['src_art_lang']['urlname'])) {
		$artLang = new cApiArticleLanguage($param['src_art_lang']['idartlang']);
		$urlname = $artLang->get('urlname');
	} else {
		$urlname = $param['src_art_lang']['urlname'];
	}

	if ($urlname) {
		ModRewrite::set_article($urlname, $param['dest_art_lang']['idart'], $param['dest_art_lang']['idlang']);
	}

	return $param;
}


/**
 * mr_build_new_url()
 *
 * Works as a wrapper for ModRewrite::build_new_url() function.
 *
 * Will be called by chain 'Contenido.Frontend.CreateURL'.
 *
 * @param   string  $url  URL to rebuild
 * @return  string        New URL
 */
function mr_build_new_url($url) {
	global $cfg;
	static $isXHTML;
	if (!isset($isXHTML)) {
		$isXHTML = (getEffectiveSetting('generator', 'xhtml', 'false') == 'false') ? false : true;
	}

	$urlDebug['in'] = $url;

	// cleanup any existing & entities
	$url = str_replace('&amp;', '&', $url);

	if ($cfg['mod_rewrite']['use'] == 1) {
		$aUrl = ModRewrite::get_client_full_url_parts($url);
		if (preg_match("/^front_content\.php(.*|.+?)/i", $aUrl['url'], $arr_hits) == 1) {
			$url = $aUrl['htmlpath'] . ModRewrite::build_new_url($arr_hits[1]);
		} else {
			$GLOBALS['mpDebug']->addDebug($url, 'mr_build_new_url() nomatch');
		}
	}

	// replace & against entity, if xhtml is to use
	if ($isXHTML) {
		$url = str_replace('&', '&amp;', $url);
	}

	$urlDebug['out'] = $url;
	$GLOBALS['mpDebug']->addDebug($urlDebug, 'mr_build_new_url() in -> out');
	return $url;
}



/**
 * mr_build_generated_code()
 *
 * Replaces existing ancors inside passed code, while rebuilding the urls.
 *
 * Will be called by chain 'Contenido.Content.conGenerateCode' or
 * 'Contenido.Frontend.HTMLCodeOutput' depening on mod_rewrite settings.
 *
 * @param   string  $code   Code to prepare
 * @return  string          New code
 */
function mr_build_generated_code($code) {
	global $cfg;

	$GLOBALS['mpDebug']->addDebug($code, 'mr_build_generated_code() in');

	// mod rewrite is activated
	if ($cfg['mod_rewrite']['use'] == 1) {
		$sseStarttime = getmicrotime();

		// edit 060603 - anchor hack
		$code = preg_replace_callback(
			"/<a([^>]*)href\s*=\s*[\"|\'][\/]#(.?|.+?)[\"|\']([^>]*)>/i", 
			create_function('$arr_matches' , 'return ModRewrite::rewrite_html_anchor($arr_matches);'),
			$code
		);

		// remove fucking tinymce single quote entities:
		$code = str_replace("&#39;", "'", $code);

		// IE hack with wrong base href interpretation
		$code = preg_replace("/([\"|\'|=])upload\/(.?|.+?)([\"|\'|>])/ie", "stripslashes('\\1${str_base_uri}upload/\\2\\3')", $code);

		// ok let it beginn, start mod rewrite class
		$code = preg_replace_callback(
			"/([\"|\'|=])front_content\.php(.?|.+?)([\"|\'|>])/i", 
			create_function('$arr_matches' , 'return $arr_matches[1] . ModRewrite::build_new_url($arr_matches[2]) . $arr_matches[3];'),
			$code
		);
		$sseEndtime = getmicrotime();
	} else {
		// anchor hack for non modrewrite websites
		$code = preg_replace_callback(
					"/<a([^>]*)href\s*=\s*[\"|\'][\/]#(.?|.+?)[\"|\']([^>]*)>/i", 
		create_function('$arr_matches' , 'return ModRewrite::contenido_html_anchor($arr_matches, $GLOBALS["is_XHTML"]);'),
		$code
		);
	}
	
	$GLOBALS['mpDebug']->addDebug($code, 'mr_build_generated_code() out');
	
	return $code;
	// print "\n\n<!-- modrewrite generation time: " . ($sseEndtime - $sseStarttime) . " seconds -->";
}


/**
 * Sets language of client, like done in front_content.php
 *
 * @param  int  $client  Client id
 */
function mr_set_client_language($client) {
	if ($GLOBALS['lang'] || (int) $GLOBALS['lang'] > 0) {
		// there is nothing to do
		return;
	}

	// use the first language of this client
	if (isset($GLOBALS['load_lang'])) {
		// load_client is set in frontend/config.php
		$GLOBALS['lang'] = $GLOBALS['load_lang'];
		return;
	}

	// try to get clients language from table
	$sql = "SELECT B.idlang FROM
                ".$GLOBALS['cfg']['tab']['clients_lang']." AS A, 
                ".$GLOBALS['cfg']['tab']['lang']." AS B
            WHERE
                A.idclient='" . ((int) $client) . "' AND 
                A.idlang=B.idlang
            LIMIT 0,1";

	if ($aData = mr_query_n_next_record($sql)) {
		$GLOBALS['lang'] = $aData['idlang'];
	}
}


/**
 * Loads Advanced Mod Rewrite configuration for passed client using serialized
 * file containing the settings.
 *
 * File is placed in /contenido/mod_rewrite/includes/and is named like
 * config.mod_rewrite_{client_id}.php.
 *
 *
 * @param  int  $clientId  Id of client
 */
function mr_load_configuration($clientId) {
	global $cfg;
	static $aLoaded;

	$clientId = (int) $clientId;
	if (!isset($aLoaded)) {
		$aLoaded = array();
	} elseif (isset($aLoaded[$clientId])) {
		return;
	}

	$options['key'] = $cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/config.mod_rewrite_' . $clientId . '.php';

	$config = ConfigFactory::get('serializer', $options);
	$mrConfig = $config->get();
	if (is_array($mrConfig)) {
	  
		// merge mod rewrite configuration with global cfg array
		$cfg = array_merge($cfg, $mrConfig);
	  
	} else {

		// couldn't load configuration, set defaults
		include_once($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/config.mod_rewrite_default.php');

	}

	$aLoaded[$clientId] = true;

}


/**
 * Database query helper. Used to execute a selece statement and to return the result of first
 * recordset.
 *
 * Minimizes following code:
 * <code>
 * // default way
 * $db  = new DB_Contenido();
 * $sql = "SELECT * FROM foo WHERE bar='foobar'";
 * $db->query($sql);
 * $db->next_record();
 * $data = $db->Record;
 *
 * // new way
 * $sql  = "SELECT * FROM foo WHERE bar='foobar'";
 * $data = mr_query_n_next_record($sql);
 * </code>
 *
 * @param   string  $query  Query to execute
 * @return  mixed   Assoziative array including recordset or null
 */
function mr_query_n_next_record($query){
	static $db;
	if (!isset($db)) {
		$db = new DB_Contenido();
	}
	$db->query($query);
	return ($db->next_record()) ? $db->Record : null;
}

