<?php
/**
 * Functions to upgrade to modrewrite, used by plugin setup
 *
 * @author      Stefan Seifarth / stese
 * @copyright    www.polycoder.de
 * @package     Contenido
 * @subpackage  ModRewrite
 */


/******************************************
* File      :   functions.mod_rewrite_setup.php
* Project   :   Contenido
* Descr     :   functions to upgrade to modrewrite
*
* Author    :   Stefan Seifarth
* Created   :   28.12.2005
* Modified  :   28.12.2005
* Modified  :   06.03.2008, Murat Purc, extracted from setup/lib directory
* 
*  www.polycoder.de
*/
error_reporting (E_ALL ^ E_NOTICE);


/**
 * upgradeModRewrite
 * 
 * handle upgrading of mod rewrite needed 
 * database table columns
 */
function upgradeModRewrite () {
    global $cfg;

    $db = new DB_Contenido();

    $sql = "ALTER TABLE " . $cfg["tab"]["art_lang"] . " ADD urlname VARCHAR( 128 ) AFTER title";
    $db->query( $sql );

    // check for empty article fields
    $sql = "SELECT idlang, title, idart FROM " . $cfg["tab"]["art_lang"] . " WHERE urlname IS NULL OR urlname = ''";
    $db->query($sql);
    while ( $db->next_record() ) {
        set_article($db->f('title'), $db->f('idart'), $db->f('idlang'));
    }

    // check for empty category fields
    $sql = "SELECT name, idcat, idlang FROM " . $cfg["tab"]["cat_lang"] . " WHERE urlname IS NULL OR urlname = ''";
    $db->query($sql);
    while ( $db->next_record() ) {
        set_category($db->f('name'), $db->f('idcat'), $db->f('idlang'));
    }

}


function uninstallModRewrite() {
    global $cfg;
    $db = new DB_Contenido();
    $sql = "ALTER TABLE " . $cfg["tab"]["art_lang"] . " DROP urlname";
    $db->query( $sql );
}


/**
 * set_article()
 *
 * set websafe name in article list
 *
 * insert new websafe name in article list 
 *
 * @param   string  original name (will be converted)
 * @param   integer current article id
 * @param   integer current language id
 * @return  boolean true if insert was successfully
 */
function set_article ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
    global $cfg;
    static $db;
    
    if (!isset($db)) {
        $db = new DB_Contenido();
    }
    
    // create websafe name
    $str_new_name = capiStrCleanURLCharacters($str_name);

    // check if websafe name already exists
    if ( in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat) ) {
        // create new websafe name if exists
        $str_new_name = capiStrCleanURLCharacters($str_name) . "_" . $int_id;
    } 

    // check again - and set name
    if (!in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat)) {
        // insert websafe name in article list
        $sql = "UPDATE " . $cfg["tab"]["art_lang"] . " SET urlname = '" . $str_new_name . "' WHERE idart = '" . $int_id . "' AND idlang = '" . $int_lang_id . "'";
        return $db->query($sql);
    } else {
        return false;
    }
} // end function




/**
 * set_category()
 *
 * set websafe name in category list
 *
 * insert new websafe name in category list 
 *
 * @param   string  original name (will be converted)
 * @param   integer current article id
 * @param   integer current language id
 * @return  boolean true if insert was successfully
 */
function set_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
    global $cfg;
    static $db;
    
    if (!isset($db)) {
        $db = new DB_Contenido();
    }
    
    // create websafe name
    $str_new_name = capiStrCleanURLCharacters($str_name);

    // check if websafe name already exists
    if (in_category($str_new_name, $int_id, $int_lang_id)) {
    // create new websafe name if exists
    $str_new_name = capiStrCleanURLCharacters($str_name) . "_" . $int_id;
    }


    // check again - and set name
    if (!in_category($str_new_name, $int_id, $int_lang_id)) {
        // insert websafe name in article list
        $sql = "UPDATE " . $cfg["tab"]["cat_lang"] . " SET urlname = '$str_new_name' WHERE idcat = '$int_id' AND idlang = '$int_lang_id'";
        return $db->query($sql);
    } else {
        return false;
    }
} // end function



    
/**
 * in_articles()
 *
 * Check articles on websafe name
 *
 * Check all articles in the current category 
 * on existing same websafe name
 *
 * @param   string  Websafe name to check
 * @param   integer current article id
 * @param   integer current language id
 * @param   integer current category id
 * @return  boolean true if websafename already exists, false if not
 */
function in_articles ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
    global $cfg;
    static $db;
    
    if (!isset($db)) {
        $db = new DB_Contenido();
    }
    
    // handle multipages
    if ($int_idcat > 0) {
        $int_category_id = $int_idcat;
    } else {
        // get category id if not set
        $sql = "SELECT idcat FROM " . $cfg["tab"]["cat_art"] . " WHERE idart = '$int_id'";
        $db->query($sql);
        $db->next_record();
        $int_category_id = ($db->f("idcat") > 0 ) ? $db->f("idcat") : "0";
    }

    $str_where = " ca.idcat = '$int_category_id' AND"
               . " al.idlang = '" . $int_lang_id . "' AND"
               . " al.urlname = '" . $str_name . "' AND"
               . " al.idart <> '$int_id'";

    // check if websafe name is in this category
    $sql = "SELECT count(al.idart) as numcats FROM " . $cfg["tab"]["art_lang"] . " al LEFT JOIN " . $cfg["tab"]["cat_art"] . " ca ON al.idart = ca.idart WHERE " . $str_where;
    $db->query($sql);
    $db->next_record();

    $int_count = $db->f("numcats");

    return ($int_count > 0) ? true : false;
} // end function



/**
 * in_categories()
 *
 * Check categories on websafe name
 *
 * Check all categories in the main parent 
 * category on existing same websafe name
 *
 * @param   string  Websafe name to check
 * @param   integer current category id
 * @param   integer current language id
 * @return  boolean true if websafename already exists, false if not
 */
function in_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
    global $cfg;
    static $db;
    
    if (!isset($db)) {
        $db = new DB_Contenido();
    }

    // get parentid
    $sql = "SELECT parentid FROM " . $cfg["tab"]["cat"] . " WHERE idcat = '$int_id'";
    $db->query($sql);
    $db->next_record();
    $int_parent_id = ($db->f("parentid") > 0 ) ? $db->f("parentid") : "0";

    $str_where = " c.parentid = '$int_parent_id' AND"
               . " cl.idlang = '" . $int_lang_id . "' AND"
               . " cl.urlname = '" . $str_name . "' AND"
               . " cl.idcat <> '$int_id'";

    // check if websafe name is in this category
    $sql = "SELECT count(cl.idcat) as numcats FROM " . $cfg["tab"]["cat_lang"] . " cl LEFT JOIN " . $cfg["tab"]["cat"] . " c ON cl.idcat = c.idcat WHERE " . $str_where;
    $db->query($sql);
    $db->next_record();

    $int_count = $db->f("numcats");

    return ($int_count > 0) ? true : false;
} // end function


?>